#include <stdio.h>

#define TAMANHO_MAXIMO 100
#define SEQUENCIA_VALIDA 1000
#define SEQUENCIA_INVALIDA 1001
#define SEQUENCIA_VITORIOSA 1002
#define SEQUENCIA_EMPATE 1003

static char *const JOGA_A = "[Joga A]";
static char *const JOGA_B = "[Joga B]";
static int JOGADOR_ATUAL = 0;
static char *const PEDIR_JOGADA = "Jogada (indice valor):\n";
static int CONTAGEM_JOGADAS = 0;
static int INSERIU_OU_REMOVEU = 0;

int lerValorDeK();

int validarK(int k);

void imprimirErroKInvalido();

int lerSequencia(int sequencia[], int tamanhoMaximo);

int calcularProduto(int sequencia[], int tamanho);

int calcularSoma(int sequencia[], int tamanho);

int valorAbsoluto(int numero);

int calcularSomaDiferencasAbsolutas(int sequencia[], int tamanho);

int validarSequencia(int sequencia[], int tamanho, int k);

int validarJogada(int sequencia[100], int tamanho, int k);

void printSequencia(const int *sequencia, int tamanhoSequencia);

void lerJogada(int sequencia[], int *tamanho, int k);

void inicializarSequencia(int sequencia[100], int K);

int processarTurno(void);

// Funções auxiliares para alínea D
static void removerElemento(int sequencia[], int *tamanho, int indice);

static void substituirElemento(int sequencia[], int indice, int valor);

static void adicionarElementoFinal(int sequencia[], int *tamanho, int valor);

static void inserirElemento(int sequencia[], int *tamanho, int indice, int valorAbsoluto);

static int testarMovimento(int sequenciaOriginal[], int tamanhoOriginal, int k, int indice, int valor);

static int jogadaArtificial(int sequencia[], int *tamanho, int k, int *indiceIA, int *valorIA);

int lerValorDeK() {
    int k;
    printf("Indique K: ");
    scanf("%d", &k);
    printf("\n");
    return k;
}

int validarK(int k) {
    return (k >= 2 && k <= 100);
}

void imprimirErroKInvalido() {
    printf("K tem de ser entre 2 e 100.\n");
}

int lerSequencia(int sequencia[], int tamanhoMaximo) {
    int numero, contador = 0;
    printf("Indique uma sequencia de numeros inteiros positivos, terminando com 0: ");
    while (contador < tamanhoMaximo) {
        scanf("%d", &numero);
        if (numero == 0) break;
        if (numero > 0) {
            sequencia[contador++] = numero;
        } else {
            printf("Numero invalido. Por favor, insira um inteiro positivo.\n");
        }
    }
    return contador;
}

int valorAbsoluto(int numero) {
    return (numero < 0) ? -numero : numero;
}

int calcularProduto(int sequencia[], int tamanho) {
    int produto = 1;
    for (int i = 0; i < tamanho; i++) {
        produto *= sequencia[i];
    }
    return produto;
}

int calcularSoma(int sequencia[], int tamanho) {
    int soma = 0;
    for (int i = 0; i < tamanho; i++) {
        soma += sequencia[i];
    }
    return soma;
}

int calcularSomaDiferencasAbsolutas(int sequencia[], int tamanho) {
    int somaDiferencas = 0;
    for (int i = 0; i < tamanho - 1; i++) {
        for (int j = i + 1; j < tamanho; j++) {
            somaDiferencas += valorAbsoluto(sequencia[i] - sequencia[j]);
        }
    }
    return somaDiferencas;
}

int validarSequencia(int sequencia[], int tamanho, int k) {
    int soma = calcularSoma(sequencia, tamanho);
    int produto = calcularProduto(sequencia, tamanho);
    return (produto >= k && soma <= k);
}

int validarJogada(int sequencia[100], int tamanho, int k) {
    if (!validarSequencia(sequencia, tamanho, k))
        return SEQUENCIA_INVALIDA;

    if (calcularSomaDiferencasAbsolutas(sequencia, tamanho) == k)
        return SEQUENCIA_VITORIOSA;

    if (CONTAGEM_JOGADAS == k)
        return SEQUENCIA_EMPATE;

    return SEQUENCIA_VALIDA;
}

void printSequencia(const int *sequencia, int tamanhoSequencia) {
    printf("Sequencia: ");
    for (int i = 0; i < tamanhoSequencia; i++) {
        printf("%d ", sequencia[i]);
    }
}

void inicializarSequencia(int sequencia[100], int K) {
    int valor = K / 2;
    sequencia[0] = valor;
    sequencia[1] = valor;
    printf("Sequencia: %d %d  %s\n", valor, valor, JOGA_A);
}

int processarTurno(void) {
    if (INSERIU_OU_REMOVEU > 1) {
        JOGADOR_ATUAL = !JOGADOR_ATUAL;
        INSERIU_OU_REMOVEU = 0;
    }
    return JOGADOR_ATUAL;
}

// Auxiliares da jogada
static void removerElemento(int sequencia[], int *tamanho, int indice) {
    for (int i = indice; i < (*tamanho) - 1; i++) {
        sequencia[i] = sequencia[i + 1];
    }
    (*tamanho)--;
    INSERIU_OU_REMOVEU++;
}

static void substituirElemento(int sequencia[], int indice, int valor) {
    sequencia[indice] = valor;
    INSERIU_OU_REMOVEU += 2;
}

static void adicionarElementoFinal(int sequencia[], int *tamanho, int valor) {
    sequencia[*tamanho] = valor;
    (*tamanho)++;
    INSERIU_OU_REMOVEU += 2;
}

static void inserirElemento(int sequencia[], int *tamanho, int indice, int valorAbsoluto) {
    for (int i = *tamanho; i > indice; i--) {
        sequencia[i] = sequencia[i - 1];
    }
    sequencia[indice] = valorAbsoluto;
    (*tamanho)++;
    INSERIU_OU_REMOVEU++;
}

//Testar um movimento hipotético sem alterar o estado real do jogo
static int testarMovimento(int sequenciaOriginal[], int tamanhoOriginal, int k, int indice, int valor) {
    int copia[TAMANHO_MAXIMO];
    for (int i = 0; i < tamanhoOriginal; i++) copia[i] = sequenciaOriginal[i];
    int tam = tamanhoOriginal;
    int antigoInseriuRemoveu = INSERIU_OU_REMOVEU;
    int antigoJogador = JOGADOR_ATUAL;
    int antigoContagem = CONTAGEM_JOGADAS;

    // Aplicar o movimento hipotético
    if (indice < 0) indice = 0;

    if (valor == 0) {
        if (indice < tam) {
            for (int i = indice; i < tam - 1; i++) {
                copia[i] = copia[i + 1];
            }
            tam--;
        }
    } else if (valor > 0) {
        if (indice < tam) {
            copia[indice] = valor;
        } else {
            copia[tam++] = valor;
        }
    } else {
        int valorAbs = -valor;
        if (indice < tam) {
            for (int i = tam; i > indice; i--) {
                copia[i] = copia[i - 1];
            }
            copia[indice] = valorAbs;
            tam++;
        } else {
            copia[tam++] = valorAbs;
        }
    }

    // Testar o resultado
    int resultado = validarJogada(copia, tam, k);

    // Restaurar variáveis globais
    INSERIU_OU_REMOVEU = antigoInseriuRemoveu;
    JOGADOR_ATUAL = antigoJogador;
    CONTAGEM_JOGADAS = antigoContagem;

    return resultado;
}

// jogadaArtificial conforme a heurística - tenta encontrar uma jogada vitoriosa ou válida dentro do limite de tentativas e dos requisitos da alinea D.
static int jogadaArtificial(int sequencia[], int *tamanho, int k, int *indiceIA, int *valorIA) {
    int soma = calcularSoma(sequencia, *tamanho);
    // Encontrar índices por valor decrescente
    // Simples: Achar o valor máximo, tentar, se não der, reduzir valor desse valor, etc.
    // Se falhar, remover esse índice do radar (fazemos um loop no max 10000 tentativas)

    // Em vez de algo complexo, iteramos no máximo *tamanho vezes procurando o maior valor não testado.
    // Guardaremos os índices já testados removendo-os de tentativas se falhar.

    int tentativas = 0;
    int usados[100] = {0}; // marca índices já tentados
    while (tentativas < 10000) {
        // Achar max valor não usado
        int maiorValor = -1;
        int maiorIndice = -1;
        for (int i = 0; i < *tamanho; i++) {
            if (!usados[i]) {
                if (sequencia[i] > maiorValor) {
                    maiorValor = sequencia[i];
                    maiorIndice = i;
                }
            }
        }
        if (maiorIndice < 0) {
            // Nenhum índice disponível
            return 0; // sem jogada
        }

        // Gerar jogadas:
        // Ordem:
        //  1. Remover (valor=0)
        //  2. Reduzir (valor=1,...,V-1)
        //  Se soma < k:
        //    3. Aumentar (valor=V+1,... V+(K - soma))
        //    4. Adicionar (-1 ... -(K - soma))
        // Tentar encontrar jogada vitoriosa primeiro, depois jogada válida com menor tamanho
        // Faremos uma lista simples: tentamos na ordem e guardamos melhor jogada.
        // Ordem tentada: remover(0), reduzir(1..V-1), aumentar(V+1..V+(k-soma)), adicionar(-1..-(k-soma))

        int valorInicial = sequencia[maiorIndice];
        int melhorTamanho = 9999;
        int melhorValor = 0;
        int melhorEhVitoria = 0;
        int limiteAumento = 0;
        int limiteAdicao = 0;
        if (soma < k) {
            limiteAumento = k - soma;
            limiteAdicao = k - soma;
        }

        // Testar remover
        {
            int res = testarMovimento(sequencia, *tamanho, k, maiorIndice, 0);
            if (res == SEQUENCIA_VITORIOSA) {
                *indiceIA = maiorIndice;
                *valorIA = 0;
                return 1;
            }
            if (res == SEQUENCIA_VALIDA) {
                // Ver tamanho resultante
                int tamRes = *tamanho - 1;
                if (tamRes < melhorTamanho && !melhorEhVitoria) {
                    melhorTamanho = tamRes;
                    melhorValor = 0;
                }
            }
        }

        // Reduzir
        for (int v = 1; v < valorInicial; v++) {
            int res = testarMovimento(sequencia, *tamanho, k, maiorIndice, v);
            if (res == SEQUENCIA_VITORIOSA) {
                *indiceIA = maiorIndice;
                *valorIA = v;
                return 1;
            }
            if (res == SEQUENCIA_VALIDA && !melhorEhVitoria) {
                // tamanho result
                int tamRes = (v > 0 && v < valorInicial) ? *tamanho : *tamanho;
                // substituição ou diminuição não altera tamanho
                if (tamRes < melhorTamanho) {
                    melhorTamanho = tamRes;
                    melhorValor = v;
                }
            }
        }

        // Aumentar
        if (limiteAumento > 0) {
            for (int v = valorInicial + 1; v <= valorInicial + limiteAumento; v++) {
                int res = testarMovimento(sequencia, *tamanho, k, maiorIndice, v);
                if (res == SEQUENCIA_VITORIOSA) {
                    *indiceIA = maiorIndice;
                    *valorIA = v;
                    return 1;
                }
                if (res == SEQUENCIA_VALIDA && !melhorEhVitoria) {
                    int tamRes = *tamanho; // substituição
                    if (tamRes < melhorTamanho) {
                        melhorTamanho = tamRes;
                        melhorValor = v;
                    }
                }
            }
        }

        // Adicionar novo número (-1 ... -(K - soma))
        if (limiteAdicao > 0) {
            for (int v = -1; v >= -limiteAdicao; v--) {
                int res = testarMovimento(sequencia, *tamanho, k, maiorIndice, v);
                if (res == SEQUENCIA_VITORIOSA) {
                    *indiceIA = maiorIndice;
                    *valorIA = v;
                    return 1;
                }
                if (res == SEQUENCIA_VALIDA && !melhorEhVitoria) {
                    // inserção aumenta tamanho
                    int tamRes = *tamanho + 1;
                    if (tamRes < melhorTamanho) {
                        melhorTamanho = tamRes;
                        melhorValor = v;
                    }
                }
            }
        }

        // Após testar todas as jogadas deste índice:
        if (melhorTamanho < 9999) {
            // Temos uma jogada válida
            *indiceIA = maiorIndice;
            *valorIA = melhorValor;
            return 1;
        }

        // Caso contrário, marcar este índice como usado e tentar outro
        usados[maiorIndice] = 1;
        tentativas++;
    }

    return 0; // não encontrou jogada
}

void lerJogada(int sequencia[], int *tamanho, int k) {
    int indice, valor;
    printf("%s", PEDIR_JOGADA);
    scanf("%d %d", &indice, &valor);

    // Jogada artificial
    if (indice == -2 && valor == -2) {
        int iA, vA;
        if (jogadaArtificial(sequencia, tamanho, k, &iA, &vA)) {
            // Aplicar a jogada artificial como se fosse a do utilizador
            indice = iA;
            valor = vA;
            printf("Jogada artificial: %d %d\n", indice, valor);
        } else {
            // Sem jogada possível, faz nada  --- pode ser um erro ??? =S
            // Aqui assumimos que se não há jogada, tentamos remover algo inválido para encerrar
            indice = 0;
            valor = 0;
            printf("Jogada artificial: 0 0 (sem opcoes)\n");
        }
    }

    if (indice < 0) indice = 0;

    if (valor == 0) {
        if (indice < *tamanho) {
            removerElemento(sequencia, tamanho, indice);
        } else {
            INSERIU_OU_REMOVEU++;
        }
    } else if (valor > 0) {
        if (indice < *tamanho) {
            substituirElemento(sequencia, indice, valor);
        } else {
            adicionarElementoFinal(sequencia, tamanho, valor);
        }
    } else {
        int valorAbs = -valor;
        if (indice < *tamanho) {
            inserirElemento(sequencia, tamanho, indice, valorAbs);
        } else {
            sequencia[*tamanho] = valorAbs;
            (*tamanho)++;
            INSERIU_OU_REMOVEU++;
        }
    }
    CONTAGEM_JOGADAS++;
}

int main() {
    int k = lerValorDeK();
    if (!validarK(k)) {
        imprimirErroKInvalido();
        return 0;
    }

    int sequencia[TAMANHO_MAXIMO];
    inicializarSequencia(sequencia, k);
    int tamanhoSequencia = 2;

    while (1) {
        lerJogada(sequencia, &tamanhoSequencia, k);
        printSequencia(sequencia, tamanhoSequencia);
        int validacao = validarJogada(sequencia, tamanhoSequencia, k);

        if (validacao == SEQUENCIA_INVALIDA) {
            printf("\nJogador %c perdeu.\n", JOGADOR_ATUAL == 0 ? 'A' : 'B');
            return 0;
        }
        if (validacao == SEQUENCIA_VITORIOSA) {
            printf("\nJogador %c ganhou.\n", JOGADOR_ATUAL == 0 ? 'A' : 'B');
            return 0;
        }
        if (validacao == SEQUENCIA_EMPATE) {
            printf("\nEmpate.\n");
            return 0;
        }

        processarTurno();
        printf(" %s\n", JOGADOR_ATUAL == 0 ? JOGA_A : JOGA_B);
    }
}
