#include <stdio.h>

#define TAMANHO_MAXIMO 100
#define SEQUENCIA_VALIDA 1000
#define SEQUENCIA_INVALIDA 1001
#define SEQUENCIA_VITORIOSA 1002
#define SEQUENCIA_EMPATE 1003

// Mensagens e estados globais
static char *const JOGA_A = "[Joga A]";
static char *const JOGA_B = "[Joga B]";
static char *const PEDIR_JOGADA = "Jogada (indice valor):\n";

static int JOGADOR_ATUAL = 0;       // 0 = A, 1 = B
static int CONTAGEM_JOGADAS = 0;
static int INSERIU_OU_REMOVEU = 0;  // Controlo para manter ou passar a vez

// Declaração de funções
int lerValorDeK();
int validarK(int k);
void imprimirErroKInvalido();
int calcularProduto(int sequencia[], int tamanho);
int calcularSoma(int sequencia[], int tamanho);
int valorAbsoluto(int numero);
int calcularSomaDiferencasAbsolutas(int sequencia[], int tamanho);
int validarSequencia(int sequencia[], int tamanho, int k);
int validarJogada(int sequencia[100], int tamanho, int k);

void printSequencia(const int *sequencia, int tamanhoSequencia);
void inicializarSequencia(int sequencia[100], int K);
void lerJogada(int sequencia[], int *tamanho);
int processarTurno(void);

// Funções auxiliares de manipulação da sequência (usadas em lerJogada)
static void removerElemento(int sequencia[], int *tamanho, int indice);
static void substituirElemento(int sequencia[], int indice, int valor);
static void adicionarElementoNoFinal(int sequencia[], int *tamanho, int valor);
static void inserirElemento(int sequencia[], int *tamanho, int indice, int valorAbsoluto);

// ------------------------ Funções Básicas ------------------------

int lerValorDeK() {
    int k;
    printf("Indique K: ");
    scanf("%d", &k);
    printf("\n");
    return k;
}

int validarK(int k) {
    return (k >= 2 && k <= 100);
}

void imprimirErroKInvalido() {
    printf("K tem de ser entre 2 e 100.\n");
}

int valorAbsoluto(int numero) {
    return (numero < 0) ? -numero : numero;
}

// ------------------------ Cálculos sobre a Sequência ------------------------

int calcularProduto(int sequencia[], int tamanho) {
    int produto = 1;
    for (int i = 0; i < tamanho; i++) {
        produto *= sequencia[i];
    }
    return produto;
}

int calcularSoma(int sequencia[], int tamanho) {
    int soma = 0;
    for (int i = 0; i < tamanho; i++) {
        soma += sequencia[i];
    }
    return soma;
}

int calcularSomaDiferencasAbsolutas(int sequencia[], int tamanho) {
    int somaDiferencas = 0;
    for (int i = 0; i < tamanho - 1; i++) {
        for (int j = i + 1; j < tamanho; j++) {
            somaDiferencas += valorAbsoluto(sequencia[i] - sequencia[j]);
        }
    }
    return somaDiferencas;
}

// ------------------------ Validação da Sequência e Jogada ------------------------

int validarSequencia(int sequencia[], int tamanho, int k) {
    int soma = calcularSoma(sequencia, tamanho);
    int produto = calcularProduto(sequencia, tamanho);
    return (produto >= k && soma <= k);
}

int validarJogada(int sequencia[100], int tamanho, int k) {
    if (!validarSequencia(sequencia, tamanho, k))
        return SEQUENCIA_INVALIDA;

    if (calcularSomaDiferencasAbsolutas(sequencia, tamanho) == k)
        return SEQUENCIA_VITORIOSA;

    if (CONTAGEM_JOGADAS == k)
        return SEQUENCIA_EMPATE;

    return SEQUENCIA_VALIDA;
}

// ------------------------ Impressão da Sequência ------------------------

void printSequencia(const int *sequencia, int tamanhoSequencia) {
    printf("Sequencia: ");
    for (int i = 0; i < tamanhoSequencia; i++) {
        printf("%d ", sequencia[i]);
    }
}

// ------------------------ Inicialização da Sequência ------------------------

void inicializarSequencia(int sequencia[100], int K) {
    int valor = K / 2;
    sequencia[0] = valor;
    sequencia[1] = valor;
    printf("Sequencia: %d %d  %s\n", valor, valor, JOGA_A);
}

// ------------------------ Operações Internas de lerJogada ------------------------

static void removerElemento(int sequencia[], int *tamanho, int indice) {
    for (int i = indice; i < (*tamanho) - 1; i++) {
        sequencia[i] = sequencia[i + 1];
    }
    (*tamanho)--;
    INSERIU_OU_REMOVEU++;
}

static void substituirElemento(int sequencia[], int indice, int valor) {
    sequencia[indice] = valor;
    INSERIU_OU_REMOVEU += 2; // Substituição conta como jogada que passa a vez
}

static void adicionarElementoNoFinal(int sequencia[], int *tamanho, int valor) {
    sequencia[*tamanho] = valor;
    (*tamanho)++;
    INSERIU_OU_REMOVEU += 2; // Adicionar no final também faz passar a vez
}

static void inserirElemento(int sequencia[], int *tamanho, int indice, int valorAbsoluto) {
    for (int i = *tamanho; i > indice; i--) {
        sequencia[i] = sequencia[i - 1];
    }
    sequencia[indice] = valorAbsoluto;
    (*tamanho)++;
    INSERIU_OU_REMOVEU++;
}

// ------------------------ Leitura da Jogada ------------------------

void lerJogada(int sequencia[], int *tamanho) {
    int indice, valor;
    printf("%s", PEDIR_JOGADA);
    scanf("%d %d", &indice, &valor);

    if (indice < 0) {
        indice = 0;
    }

    if (valor == 0) {
        // Remoção
        if (indice < *tamanho) {
            removerElemento(sequencia, tamanho, indice);
        } else {
            // Índice >= tamanho: não faz nada na sequência, mas conta como inseriu/ removeu
            INSERIU_OU_REMOVEU++;
        }
    } else if (valor > 0) {
        // Substituição ou Adição no final
        if (indice < *tamanho) {
            substituirElemento(sequencia, indice, valor);
        } else {
            adicionarElementoNoFinal(sequencia, tamanho, valor);
        }
    } else {
        // Inserção de valor absoluto
        int valorAbs = -valor;
        if (indice < *tamanho) {
            inserirElemento(sequencia, tamanho, indice, valorAbs);
        } else {
            // Inserir no final
            sequencia[*tamanho] = valorAbs;
            (*tamanho)++;
            INSERIU_OU_REMOVEU++;
        }
    }

    CONTAGEM_JOGADAS++;
}

// ------------------------ Processamento do Turno ------------------------

int processarTurno(void) {
    // Se INSERIU_OU_REMOVEU > 1, significa que a jogada foi de substituição ou adição,
    // ou que o jogador já manteve a vez após inserir/remover uma vez. Então passa a vez.
    if (INSERIU_OU_REMOVEU > 1) {
        JOGADOR_ATUAL = !JOGADOR_ATUAL;
        INSERIU_OU_REMOVEU = 0;
    }
    return JOGADOR_ATUAL;
}

// ------------------------ Função Principal ------------------------

int main() {
    int k = lerValorDeK();

    if (!validarK(k)) {
        imprimirErroKInvalido();
        return 0;
    }

    int sequencia[TAMANHO_MAXIMO];
    inicializarSequencia(sequencia, k);

    int tamanhoSequencia = 2;

    while (1) {
        lerJogada(sequencia, &tamanhoSequencia);
        printSequencia(sequencia, tamanhoSequencia);

        int estadoSequencia = validarJogada(sequencia, tamanhoSequencia, k);

        if (estadoSequencia == SEQUENCIA_INVALIDA) {
            printf("\nJogador %c perdeu.\n", (JOGADOR_ATUAL == 0) ? 'A' : 'B');
            return 0;
        }

        if (estadoSequencia == SEQUENCIA_VITORIOSA) {
            printf("\nJogador %c ganhou.\n", (JOGADOR_ATUAL == 0) ? 'A' : 'B');
            return 0;
        }

        if (estadoSequencia == SEQUENCIA_EMPATE) {
            printf("\nEmpate.\n");
            return 0;
        }

        processarTurno();
        printf(" %s\n", (JOGADOR_ATUAL == 0) ? JOGA_A : JOGA_B);
    }
}
